@@ -1,5 +1,5 @@
 cd
 wget http://download.netsurf-browser.org/libs/releases/buildsystem-1.10.tar.gz
 tar -xf buildsystem-1.10.tar.gz
-make -C buildsystem-1.10 install PREFIX=$HOME/64
+make -C buildsystem-1.10 install PREFIX=/opt/elinks
 
