@@ -1,13 +1,13 @@
 #!/bin/sh
 
-rm -rf /root/tmp/builddir
+rm -rf /tmp/builddir
 
-LIBRARY_PATH="/usr/local/lib" \
-PKG_CONFIG_PATH="/usr/local/lib/pkgconfig" \
-LDFLAGS="-L/usr/local/lib" \
-CFLAGS="-O2 -I/usr/local/include -static -no-pie" \
-CXXFLAGS="-O2 -I/usr/local/include -static -no-pie" \
-meson setup /root/tmp/builddir \
+PREFIX=/opt/elinks
+DESTDIR=$HOME/elinks-bin
+
+CFLAGS="-O2 -static -no-pie" \
+CXXFLAGS="-O2 -static -no-pie" \
+meson setup /tmp/builddir \
 -D88-colors=true \
 -D256-colors=true \
 -Dbacktrace=false \
@@ -17,22 +17,28 @@ meson setup /root/tmp/builddir \
 -Dcgi=true \
 -Dcss=true \
 -Ddgi=true \
+-Ddoc=false \
 -Dexmode=true \
 -Dfastmem=true \
 -Dfsp=false \
 -Dfsp2=true \
 -Dgemini=true \
--Dgettext=false \
+-Dgettext=true \
 -Dgnutls=false \
 -Dgopher=true \
 -Dgpm=false \
 -Dguile=false \
+-Dhtmldoc=false \
+-Dhtml-highlight=true \
 -Didn=true \
 -Dipv6=true \
+-Dkitty=true \
+-Dlibavif=false \
 -Dlibcss=true \
 -Dlibcurl=true \
 -Dlibev=false \
 -Dlibevent=true \
+-Dlibwebp=true \
 -Dluapkg='luajit' \
 -Dlzma=true \
 -Dmailcap=true \
@@ -40,14 +46,17 @@ meson setup /root/tmp/builddir \
 -Dnls=true \
 -Dnntp=true \
 -Dopenssl=true \
+-Dpdfdoc=false \
 -Dperl=false \
+-Dprefix=$PREFIX \
 -Dpython=false \
 -Dquickjs=false \
 -Druby=false \
 -Dsm-scripting=false \
+-Dspartan=true \
 -Dspidermonkey=false \
 -Dstatic=true \
--Dterminfo=false \
+-Dterminfo=true \
 -Dtest=false \
 -Dtre=true \
 -Dtrue-color=true \
@@ -58,8 +67,11 @@ meson setup /root/tmp/builddir \
 -Dzlib=true \
 -Dzstd=true || exit 1
 
-meson compile -C /root/tmp/builddir || exit 2
+meson compile -C /tmp/builddir || exit 2
+mkdir -p $DESTDIR
 
-strip /root/tmp/builddir/src/elinks || exit 3
+meson install -C /tmp/builddir --destdir $DESTDIR || exit 3
 
-upx /root/tmp/builddir/src/elinks || exit 4
+mv $DESTDIR/$PREFIX/bin/elinks $DESTDIR/$PREFIX/bin/elinks-lite || exit 3
+strip $DESTDIR/$PREFIX/bin/elinks-lite || exit 4
+upx $DESTDIR/$PREFIX/bin/elinks-lite || exit 5
