@@ -1,10 +1,13 @@
 #!/bin/sh
 
-rm -rf /root/tmp/builddir_js
+rm -rf /tmp/builddir_js
+
+PREFIX=/opt/elinks
+DESTDIR=$HOME/elinks-bin
 
 CFLAGS="-O2 -static -no-pie" \
 CXXFLAGS="-O2 -static -no-pie" \
-meson setup /root/tmp/builddir_js \
+meson setup /tmp/builddir_js \
 -D88-colors=true \
 -D256-colors=true \
 -Dbacktrace=false \
@@ -14,22 +17,28 @@ meson setup /root/tmp/builddir_js \
 -Dcgi=true \
 -Dcss=true \
 -Ddgi=true \
+-Ddoc=false \
 -Dexmode=true \
 -Dfastmem=true \
 -Dfsp=false \
 -Dfsp2=true \
 -Dgemini=true \
--Dgettext=false \
+-Dgettext=true \
 -Dgnutls=false \
 -Dgopher=true \
 -Dgpm=false \
 -Dguile=false \
+-Dhtmldoc=false \
+-Dhtml-highlight=true \
 -Didn=true \
 -Dipv6=true \
+-Dkitty=true \
+-Dlibavif=false \
 -Dlibcss=true \
 -Dlibcurl=true \
 -Dlibev=false \
 -Dlibevent=true \
+-Dlibwebp=true \
 -Dluapkg='luajit' \
 -Dlzma=true \
 -Dmailcap=true \
@@ -37,14 +46,17 @@ meson setup /root/tmp/builddir_js \
 -Dnls=true \
 -Dnntp=true \
 -Dopenssl=true \
+-Dpdfdoc=false \
 -Dperl=false \
+-Dprefix=$PREFIX \
 -Dpython=false \
 -Dquickjs=true \
 -Druby=false \
 -Dsm-scripting=false \
+-Dspartan=true \
 -Dspidermonkey=false \
 -Dstatic=true \
--Dterminfo=false \
+-Dterminfo=true \
 -Dtest=false \
 -Dtre=true \
 -Dtrue-color=true \
@@ -55,8 +67,11 @@ meson setup /root/tmp/builddir_js \
 -Dzlib=true \
 -Dzstd=true || exit 1
 
-meson compile -j $(($(nproc) - 1)) -C /root/tmp/builddir_js || exit 2
+meson compile -C /tmp/builddir_js || exit 2
 
-strip /root/tmp/builddir_js/src/elinks || exit 3
+mkdir -p $DESTDIR
 
-upx /root/tmp/builddir_js/src/elinks || exit 4
+meson install -C /tmp/builddir_js --destdir $DESTDIR || exit 3
+
+strip $DESTDIR/$PREFIX/bin/elinks || exit 4
+upx $DESTDIR/$PREFIX/bin/elinks || exit 5
