@@ -92,6 +92,7 @@ static int readok = 1;
 static void
 read_bookmarks_xbel(FILE *f)
 {
+	ELOG
 	char in_buffer[BUFSIZ];
 	XML_Parser p;
 	int done = 0;
@@ -102,7 +103,7 @@ read_bookmarks_xbel(FILE *f)
 
 	p = XML_ParserCreate(NULL);
 	if (!p) {
-		ERROR(gettext("read_bookmarks_xbel(): Error in XML_ParserCreate()"));
+		EL_ERROR(gettext("read_bookmarks_xbel(): Error in XML_ParserCreate()"));
 		return;
 	}
 
@@ -113,7 +114,7 @@ read_bookmarks_xbel(FILE *f)
 		size_t len = fread(in_buffer, 1, BUFSIZ, f);
 
 		if (ferror(f)) {
-			ERROR(gettext("read_bookmarks_xbel(): Error reading %s"),
+			EL_ERROR(gettext("read_bookmarks_xbel(): Error reading %s"),
 			      filename_bookmarks_xbel(0));
 			err = 1;
 		} else {
@@ -149,6 +150,7 @@ static void
 write_bookmarks_xbel(struct secure_save_info *ssi,
 		     LIST_OF(struct bookmarks) *bookmarks_list)
 {
+	ELOG
 	int folder_state = get_opt_bool("bookmarks.folder_state", NULL);
 	/* We check for readok in filename_bookmarks_xbel(). */
 
@@ -168,6 +170,7 @@ write_bookmarks_xbel(struct secure_save_info *ssi,
 static const char *
 filename_bookmarks_xbel(int writing)
 {
+	ELOG
 	if (writing && !readok) return NULL;
 	return BOOKMARKS_XBEL_FILENAME;
 }
@@ -175,6 +178,7 @@ filename_bookmarks_xbel(int writing)
 static void
 indentation(struct secure_save_info *ssi, int num)
 {
+	ELOG
 	int i;
 
 	for (i = 0; i < num; i++)
@@ -184,6 +188,7 @@ indentation(struct secure_save_info *ssi, int num)
 static void
 print_xml_entities(struct secure_save_info *ssi, const char *str)
 {
+	ELOG
 	struct string entitized = NULL_STRING;
 
 	if (init_string(&entitized)
@@ -202,6 +207,7 @@ write_bookmarks_list(struct secure_save_info *ssi,
 		     LIST_OF(struct bookmark) *bookmarks_list,
 		     int n, int folder_state)
 {
+	ELOG
 	struct bookmark *bm;
 
 	foreach (bm, *bookmarks_list) {
@@ -226,12 +232,29 @@ write_bookmarks_list(struct secure_save_info *ssi,
 			secure_fputs(ssi, "</folder>\n\n");
 
 		} else if (bm->box_item->type == BI_LEAF) {
-
+			char *pos = strchr(bm->url, POSITION_CHAR);
 			secure_fputs(ssi, "<bookmark href=\"");
+
+			if (pos) {
+				*pos = '\0';
+			}
 			print_xml_entities(ssi, bm->url);
 			secure_fputs(ssi, "\">\n");
-
 			indentation(ssi, n + 2);
+
+			if (pos) {
+				secure_fputs(ssi, "<info>\n");
+				indentation(ssi, n + 3);
+				secure_fputs(ssi, "<metadata owner=\"http://elinks.cz\">\n");
+				indentation(ssi, n + 4);
+				secure_fputs(ssi, "<elinks:saved-position value=\"");
+				print_xml_entities(ssi, pos + 1);
+				secure_fputs(ssi, "\"/>\n");
+				indentation(ssi, n + 3);
+				secure_fputs(ssi, "</metadata>\n");
+				indentation(ssi, n + 2);
+				secure_fputs(ssi, "</info>\n");
+			}
 			secure_fputs(ssi, "<title>");
 			print_xml_entities(ssi, bm->title);
 			secure_fputs(ssi, "</title>\n");
@@ -248,6 +271,7 @@ write_bookmarks_list(struct secure_save_info *ssi,
 static void
 on_element_open(void *data, const char *name, const char **attr)
 {
+	ELOG
 	struct tree_node *node;
 
 	node = new_node(current_node);
@@ -298,12 +322,14 @@ on_element_open(void *data, const char *name, const char **attr)
 static void
 on_element_close(void *data, const char *name)
 {
+	ELOG
 	current_node = current_node->parent;
 }
 
 static char *
 delete_whites(const char *s)
 {
+	ELOG
 	char *r;
 	int last_was_space = 0, c = 0, i;
 	int len = strlen(s);
@@ -344,6 +370,7 @@ delete_whites(const char *s)
 static void
 on_text(void *data, const XML_Char *text, int len)
 {
+	ELOG
 	char *tmp;
 	int len2 = 0;
 
@@ -363,6 +390,27 @@ on_text(void *data, const XML_Char *text, int len)
 	}
 }
 
+static const char *
+get_saved_position(struct tree_node *bookmarknode)
+{
+	struct tree_node *info = get_child(bookmarknode, "info");
+
+	if (!info) {
+		return NULL;
+	}
+	struct tree_node *metadata = get_child(info, "metadata");
+
+	if (!metadata) {
+		return NULL;
+	}
+	struct tree_node *elinks_saved_pos = get_child(metadata, "elinks:saved-position");
+
+	if (!elinks_saved_pos) {
+		return NULL;
+	}
+	return get_attribute_value(elinks_saved_pos, "value");
+}
+
 /* xbel_tree_to_bookmarks_list: returns 0 on fail,
  *				      1 on success */
 static int
@@ -370,17 +418,26 @@ xbeltree_to_bookmarks_list(const struct read_bookmarks_xbel *preload,
 			   struct tree_node *node,
 			   struct bookmark *current_parent)
 {
+	ELOG
 	struct bookmark *tmp;
 	struct tree_node *title;
 	static struct bookmark *lastbm;
 
 	while (node) {
 		if (!strcmp(node->name, "bookmark")) {
+			char *href_with_pos = NULL;
 			char *href;
 
 			title = get_child(node, "title");
 			href = get_attribute_value(node, "href");
 
+			if (href) {
+				const char *pos = get_saved_position(node);
+
+				if (pos) {
+					href = href_with_pos = straconcat(href, POSITION_CHAR_S, pos, NULL);
+				}
+			}
 			intl_set_charset_by_index(preload->utf8_cp);
 			tmp = add_bookmark(current_parent, 0,
 					   /* The <title> element is optional */
@@ -393,6 +450,8 @@ xbeltree_to_bookmarks_list(const struct read_bookmarks_xbel *preload,
 					   href ? href
 						: (char *) gettext("No URL"));
 
+			mem_free_if(href_with_pos);
+
 			/* Out of memory */
 			if (!tmp) return 0;
 
@@ -456,6 +515,7 @@ xbeltree_to_bookmarks_list(const struct read_bookmarks_xbel *preload,
 static void
 free_xbeltree(struct tree_node *node)
 {
+	ELOG
 	struct tree_node *next_node;
 
 	while (node) {
@@ -473,6 +533,7 @@ free_xbeltree(struct tree_node *node)
 static struct tree_node *
 get_child(struct tree_node *node, const char *name)
 {
+	ELOG
 	struct tree_node *ret;
 
 	if (!node) return NULL;
@@ -492,6 +553,7 @@ get_child(struct tree_node *node, const char *name)
 static char *
 get_attribute_value(struct tree_node *node, const char *name)
 {
+	ELOG
 	struct attributes *attribute;
 
 	foreach (attribute, node->attrs) {
@@ -506,6 +568,7 @@ get_attribute_value(struct tree_node *node, const char *name)
 static struct tree_node *
 new_node(struct tree_node *parent)
 {
+	ELOG
 	struct tree_node *node;
 
 	node = (struct tree_node *)mem_calloc(1, sizeof(*node));
@@ -520,6 +583,7 @@ new_node(struct tree_node *parent)
 static void
 free_node(struct tree_node *node)
 {
+	ELOG
 	struct attributes *attribute;
 
 	foreach (attribute, node->attrs) {
