@@ -9,6 +9,7 @@
 extern "C" {
 #endif
 
+struct el_string;
 struct listbox_item;
 struct uri;
 
@@ -60,6 +61,12 @@ struct cache_entry {
 	struct JSObject *jsobject;      /* Instance of cache_entry_class */
 #endif
 
+#ifdef CONFIG_KITTY
+	struct el_string *pixels;
+	int width;
+	int height;
+	int number;
+#endif
 	timeval_T max_age;		/* Expiration time */
 
 	unsigned int expire:1;		/* Whether to honour max_age */
@@ -74,6 +81,7 @@ struct cache_entry {
 	unsigned int gc_target:1;	/* The GC touch of death */
 	unsigned int cgi:1;		/* Is a CGI output? */
 	unsigned int integrity_valid:1; /* Was integrity checked and valid */
+	unsigned int compressed:1; /* Is kitty zlib? */
 
 	cache_mode_T cache_mode;	/* Reload condition */
 };
