@@ -51,6 +51,7 @@
 
 #include "cache/cache.h"
 #include "config/options.h"
+#include "document/docdata.h"
 #include "document/document.h"
 #include "document/forms.h"
 #include "document/html/frames.h"
@@ -76,6 +77,9 @@
 #include "network/dns.h"
 #include "protocol/uri.h"
 #include "terminal/draw.h"
+#ifdef CONFIG_KITTY
+#include "terminal/kitty.h"
+#endif
 #ifdef CONFIG_LIBSIXEL
 #include "terminal/sixel.h"
 #endif
@@ -113,6 +117,7 @@ static void add_to_document_list(LIST_OF(struct document_list) *list, struct doc
 static void
 remove_document_from_format_cache(struct document *document)
 {
+	ELOG
 	struct document_list *item;
 
 	foreach (item, format_cache) {
@@ -127,6 +132,7 @@ remove_document_from_format_cache(struct document *document)
 static void
 move_document_to_top_of_format_cache(struct document *document)
 {
+	ELOG
 	struct document_list *item;
 
 	foreach (item, format_cache) {
@@ -141,6 +147,7 @@ move_document_to_top_of_format_cache(struct document *document)
 static void
 dump_format_cache(int line)
 {
+	ELOG
 	fprintf(stderr, "line=%d size=%ld", line, format_cache.size());
 	for (auto it = format_cache.begin(); it != format_cache.end(); it++) {
 		fprintf(stderr, " %p", *it);
@@ -154,6 +161,7 @@ dump_format_cache(int line)
 static void
 found_dns(void *data, struct sockaddr_storage *addr, int addrlen)
 {
+	ELOG
 	char buf[64];
 	const char *res;
 	struct sockaddr *s;
@@ -177,6 +185,7 @@ found_dns(void *data, struct sockaddr_storage *addr, int addrlen)
 static void
 get_ip(struct document *document)
 {
+	ELOG
 #ifdef HAVE_INET_NTOP
 	struct uri *uri = document->uri;
 	char *host = get_uri_string(uri, URI_DNS_HOST);
@@ -191,6 +200,7 @@ get_ip(struct document *document)
 struct document *
 init_document(struct cache_entry *cached, struct document_options *options)
 {
+	ELOG
 	struct document *document = (struct document *)mem_calloc(1, sizeof(*document));
 
 	if (!document) return NULL;
@@ -206,6 +216,7 @@ init_document(struct cache_entry *cached, struct document_options *options)
 	init_list(document->forms);
 	init_list(document->tags);
 	init_list(document->nodes);
+	init_list(document->iframes);
 
 #ifdef CONFIG_LIBDOM
 	init_string(&document->text);
@@ -215,6 +226,10 @@ init_document(struct cache_entry *cached, struct document_options *options)
 	init_list(document->onload_snippets);
 #endif
 
+#ifdef CONFIG_KITTY
+	init_list(document->k_images);
+#endif
+
 #ifdef CONFIG_LIBSIXEL
 	init_list(document->images);
 #endif
@@ -231,6 +246,7 @@ init_document(struct cache_entry *cached, struct document_options *options)
 static void
 free_frameset_desc(struct frameset_desc *frameset_desc)
 {
+	ELOG
 	int i;
 
 	for (i = 0; i < frameset_desc->n; i++) {
@@ -249,6 +265,7 @@ free_frameset_desc(struct frameset_desc *frameset_desc)
 static void
 free_iframeset_desc(struct iframeset_desc *iframeset_desc)
 {
+	ELOG
 	int i;
 
 	for (i = 0; i < iframeset_desc->n; i++) {
@@ -268,6 +285,7 @@ free_iframeset_desc(struct iframeset_desc *iframeset_desc)
 void
 done_link_members(struct link *link)
 {
+	ELOG
 	if (link->event_hooks) {
 		struct script_event_hook *evhook, *safety;
 
@@ -277,7 +295,9 @@ done_link_members(struct link *link)
 		}
 		mem_free(link->event_hooks);
 	}
-	mem_free_if(get_link_name(link));
+	if (!link_is_form(link)) {
+		mem_free_if(link->data.name);
+	}
 	mem_free_if(link->where);
 	mem_free_if(link->target);
 	mem_free_if(link->title);
@@ -285,9 +305,78 @@ done_link_members(struct link *link)
 	mem_free_if(link->points);
 }
 
+static void
+copy_link(struct link *dest, struct link *src)
+{
+	ELOG
+	dest->accesskey = src->accesskey;
+	dest->type = src->type;
+
+#ifdef CONFIG_ECMASCRIPT
+	dest->element_offset = src->element_offset; // TODO
+#endif
+	dest->where = null_or_stracpy(src->where);
+	dest->target = null_or_stracpy(src->target);
+	dest->where_img = null_or_stracpy(src->where_img);
+
+	dest->title = null_or_stracpy(src->title);
+	dest->npoints = src->npoints;
+
+	dest->points = mem_calloc(dest->npoints, sizeof(struct point));
+
+	if (dest->points) {
+		memcpy(dest->points, src->points, dest->npoints * (sizeof(struct point)));
+	}
+	dest->number = src->number;
+	dest->tabindex = src->tabindex;
+	dest->color = src->color;
+	dest->event_hooks = NULL;
+
+	if (src->event_hooks) {
+		dest->event_hooks = (LIST_OF(struct script_event_hook) *)mem_calloc(1, sizeof(*dest->event_hooks));
+
+		if (dest->event_hooks) {
+			init_list(*dest->event_hooks);
+			struct script_event_hook *evhook;
+
+			foreach (evhook, *src->event_hooks) {
+				if (!evhook->src) {
+					continue;
+				}
+				struct script_event_hook *copy_evhook = (struct script_event_hook *)mem_calloc(1, sizeof(*copy_evhook));
+				if (!copy_evhook) {
+					continue;
+				}
+				copy_evhook->type = evhook->type;
+				copy_evhook->src = stracpy(evhook->src);
+				add_to_list(*(dest->event_hooks), copy_evhook);
+			}
+		}
+	}
+
+	if (!link_is_form(src)) {
+		dest->data.name = null_or_stracpy(src->data.name);
+	} else {
+		dest->data = src->data;
+	}
+}
+
+static void
+copy_line(struct line *dest, struct line *src)
+{
+	ELOG
+	dest->length = src->length;
+	dest->ch.chars = mem_alloc(dest->length * sizeof(struct screen_char));
+
+	if (dest->ch.chars) {
+		memcpy(dest->ch.chars, src->ch.chars, dest->length * sizeof(struct screen_char));
+	}
+}
+
 void
 reset_document(struct document *document)
 {
+	ELOG
 	assert(document);
 	if_assert_failed return;
 
@@ -325,6 +414,7 @@ reset_document(struct document *document)
 		mem_free_set(&document->links, NULL);
 		document->nlinks = 0;
 	}
+	mem_free_set(&document->reverse_link_lookup, NULL);
 
 	if (document->data) {
 		int pos;
@@ -352,10 +442,12 @@ reset_document(struct document *document)
 	free_ecmascript_string_list(&document->onload_snippets);
 	free_uri_list(&document->ecmascript_imports);
 	mem_free_set(&document->body_onkeypress, NULL);
+	mem_free_set(&document->offset_linknum, NULL);
 #endif
 
 	free_list(document->tags);
 	free_list(document->nodes);
+	free_list(document->iframes);
 
 	mem_free_set(&document->search, NULL);
 	mem_free_set(&document->slines1, NULL);
@@ -366,6 +458,7 @@ reset_document(struct document *document)
 void
 done_document(struct document *document)
 {
+	ELOG
 	assert(document);
 	if_assert_failed return;
 
@@ -380,7 +473,7 @@ done_document(struct document *document)
 	mem_free_if(document->ip);
 	mem_free_if(document->title);
 	if (document->frame_desc) free_frameset_desc(document->frame_desc);
-	if (document->iframe_desc) free_iframeset_desc(document->iframe_desc);
+	if (document->iframeset_desc) free_iframeset_desc(document->iframeset_desc);
 	if (document->refresh) done_document_refresh(document->refresh);
 
 	if (document->links) {
@@ -391,6 +484,7 @@ done_document(struct document *document)
 
 		mem_free(document->links);
 	}
+	mem_free_set(&document->reverse_link_lookup, NULL);
 
 	if (document->data) {
 		int pos;
@@ -409,12 +503,22 @@ done_document(struct document *document)
 		done_form((struct form *)document->forms.next);
 	}
 
+#ifdef CONFIG_KITTY
+	while (!list_empty(document->k_images)) {
+		delete_k_image((struct k_image *)document->k_images.next);
+	}
+#endif
+
 #ifdef CONFIG_LIBSIXEL
 	while (!list_empty(document->images)) {
 		delete_image((struct image *)document->images.next);
 	}
 #endif
 
+#if defined(CONFIG_KITTY) || defined(CONFIG_LIBSIXEL)
+	free_uri_list(&document->image_uris);
+#endif
+
 #ifdef CONFIG_CSS
 	free_uri_list(&document->css_imports);
 #endif
@@ -422,6 +526,7 @@ done_document(struct document *document)
 	free_ecmascript_string_list(&document->onload_snippets);
 	free_uri_list(&document->ecmascript_imports);
 	mem_free_if(document->body_onkeypress);
+	mem_free_if(document->offset_linknum);
 #endif
 
 #ifdef CONFIG_LIBDOM
@@ -438,6 +543,7 @@ done_document(struct document *document)
 #endif
 	free_list(document->tags);
 	free_list(document->nodes);
+	free_list(document->iframes);
 
 	mem_free_if(document->search);
 	mem_free_if(document->slines1);
@@ -451,6 +557,7 @@ done_document(struct document *document)
 void
 release_document(struct document *document)
 {
+	ELOG
 	assert(document);
 	if_assert_failed return;
 
@@ -464,6 +571,7 @@ release_document(struct document *document)
 int
 find_tag(struct document *document, char *name, int namelen)
 {
+	ELOG
 	struct tag *tag;
 
 	foreach (tag, document->tags)
@@ -482,6 +590,7 @@ find_tag(struct document *document, char *name, int namelen)
 unsigned long
 get_document_css_magic(struct document *document)
 {
+	ELOG
 	unsigned long css_magic = 0;
 	struct uri *uri;
 	int index;
@@ -492,6 +601,14 @@ get_document_css_magic(struct document *document)
 		if (cached) css_magic += cached->cache_id + cached->data_size;
 	}
 
+#if defined(CONFIG_KITTY) || defined(CONFIG_LIBSIXEL)
+	foreach_uri (uri, index, &document->image_uris) {
+		struct cache_entry *cached = find_in_cache(uri);
+
+		if (cached) css_magic += cached->cache_id + cached->data_size;
+	}
+#endif
+
 	return css_magic;
 }
 
@@ -504,6 +621,7 @@ get_document_css_magic(struct document *document)
 void
 update_cached_document_options(struct session *ses)
 {
+	ELOG
 	struct document *document;
 	struct active_link_options active_link;
 	struct document_list *item;
@@ -527,6 +645,7 @@ update_cached_document_options(struct session *ses)
 static void
 add_to_document_list(LIST_OF(struct document_list) *list, struct document *document)
 {
+	ELOG
 	struct document_list *item = (struct document_list *)mem_alloc(sizeof(*item));
 
 	if (!item) {
@@ -539,6 +658,7 @@ add_to_document_list(LIST_OF(struct document_list) *list, struct document *docum
 struct document *
 get_cached_document(struct cache_entry *cached, struct document_options *options)
 {
+	ELOG
 	struct document *ret = NULL;
 	struct document_list *item, *it;
 	INIT_LIST_OF(struct document_list, to_remove);
@@ -580,6 +700,7 @@ get_cached_document(struct cache_entry *cached, struct document_options *options
 void
 shrink_format_cache(int whole)
 {
+	ELOG
 	struct document *document;
 	struct document_list *item, *it;
 	int format_cache_size = get_opt_int("document.cache.format.size", NULL);
@@ -635,12 +756,14 @@ shrink_format_cache(int whole)
 int
 get_format_cache_size(void)
 {
+	ELOG
 	return list_size(&format_cache);
 }
 
 int
 get_format_cache_used_count(void)
 {
+	ELOG
 	struct document_list *it;
 	struct document *document;
 	int i = 0;
@@ -655,6 +778,7 @@ get_format_cache_used_count(void)
 int
 get_format_cache_refresh_count(void)
 {
+	ELOG
 	struct document_list *it;
 	struct document *document;
 	int i = 0;
@@ -673,34 +797,281 @@ get_format_cache_refresh_count(void)
 static void
 init_documents(struct module *module)
 {
+	ELOG
 	init_tags_lookup();
 }
 
 static void
 done_documents(struct module *module)
 {
+	ELOG
 	free_tags_lookup();
 	free_table_cache();
 }
 
 #ifdef CONFIG_ECMASCRIPT
+
+/* comparison function for qsort() */
+static int
+comp_offset(const void *v1, const void *v2)
+{
+	ELOG
+	const struct offset_linknum *l1 = (const struct offset_linknum *)v1, *l2 = (const struct offset_linknum *)v2;
+
+	assert(l1 && l2);
+	if_assert_failed return 0;
+
+	if (l1->offset < l2->offset) {
+		return -1;
+	}
+
+	if (l1->offset > l2->offset) {
+		return 1;
+	}
+
+	return 0;
+}
+
+static void
+sort_offset(struct document *document)
+{
+	ELOG
+	if (!document->nlinks) {
+		return;
+	}
+	mem_free_set(&document->offset_linknum, malloc(document->nlinks * sizeof(struct offset_linknum)));
+
+	if (!document->offset_linknum) {
+		return;
+	}
+	int i;
+
+	for (i = 0; i < document->nlinks; i++) {
+		document->offset_linknum[i].offset = document->links[i].element_offset;
+		document->offset_linknum[i].linknum = i;
+	}
+	qsort(document->offset_linknum, document->nlinks, sizeof(struct offset_linknum), comp_offset);
+	document->offset_sorted = 1;
+}
+
 int
 get_link_number_by_offset(struct document *document, int offset)
 {
-	int link;
+	ELOG
+	if (!document->links_sorted) {
+		sort_links(document);
+	}
 
-	if (!document->links_sorted) sort_links(document);
+	if (!document->offset_sorted) {
+		sort_offset(document);
 
-	for (link = 0; link < document->nlinks; link++) {
-		if (document->links[link].element_offset == offset) {
-			return link;
+		if (!document->offset_sorted) {
+			return -1;
+		}
+	}
+	struct offset_linknum key = { .offset = offset, .linknum = -1 };
+	struct offset_linknum *item = (struct offset_linknum *)bsearch(&key, document->offset_linknum, document->nlinks, sizeof(*item), comp_offset);
+
+	return item ? item->linknum : -1;
+}
+#endif
+
+void
+insert_document_into_document(struct document *dest, struct document *src, int y)
+{
+	ELOG
+	if (!dest || !src || !src->height) {
+		return;
+	}
+	if (y > dest->height) {
+		y = dest->height;
+	}
+	if (!ALIGN_LINES(&dest->data, dest->height, dest->height + src->height)) {
+		return;
+	}
+	memmove(&dest->data[y + src->height], &dest->data[y], (dest->height - y) * sizeof(struct line));
+
+	int i;
+	for (i = 0; i < src->height; i++) {
+		copy_line(&dest->data[y + i], &src->data[i]);
+	}
+
+	if (!ALIGN_LINK(&dest->links, dest->nlinks, dest->nlinks + src->nlinks)) {
+		return;
+	}
+
+	/* old links */
+	int tomove = dest->nlinks;
+	for (i = 0; i < dest->nlinks; i++) {
+		struct link *link = &dest->links[i];
+		int c;
+		int found = 0;
+
+		for (c = 0; c < link->npoints; c++) {
+			if (link->points[c].y < y) {
+				continue;
+			};
+			link->points[c].y += src->height;
+			found = 1;
+			if (i < tomove) {
+				tomove = i;
+			}
+		}
+		if (found) {
+			link->number += src->nlinks;
+		}
+	}
+	/* new links */
+	memmove(&dest->links[tomove + src->nlinks], &dest->links[tomove], (dest->nlinks - tomove) * sizeof(struct link));
+
+	for (i = 0; i < src->nlinks; i++) {
+		copy_link(&dest->links[tomove + i], &src->links[i]);
+	}
+	//memcpy(&dest->links[tomove], &src->links[0], src->nlinks * sizeof(struct link));
+
+	for (i = 0; i < src->nlinks; i++) {
+		struct link *link = &dest->links[i + tomove];
+		int c;
+
+		for (c = 0; c < link->npoints; c++) {
+			link->points[c].y += y;
+		}
+		link->number += tomove;
+	}
+
+	/* old images */
+#ifdef CONFIG_KITTY
+	struct k_image *k_im;
+	foreach (k_im, dest->k_images) {
+		if (k_im->cy < y) {
+			continue;
+		};
+		k_im->cy += src->height;
+	}
+	/* new images */
+	foreach (k_im, src->k_images) {
+		struct k_image *imcopy = mem_calloc(1, sizeof(*imcopy));
+
+		if (!imcopy) {
+			continue;
+		}
+		copy_struct(imcopy, k_im);
+		imcopy->cy += y;
+		el_string_ref(imcopy->pixels);
+		add_to_list(dest->k_images, imcopy);
+	}
+#endif
+
+	/* old images */
+#ifdef CONFIG_LIBSIXEL
+	struct image *im;
+	foreach (im, dest->images) {
+		if (im->cy < y) {
+			continue;
+		};
+		im->cy += src->height;
+	}
+	/* new images */
+	foreach (im, src->images) {
+		struct image *imcopy = mem_calloc(1, sizeof(*imcopy));
+
+		if (!imcopy) {
+			continue;
+		}
+		copy_struct(imcopy, im);
+		imcopy->cy += y;
+		add_to_list(dest->images, imcopy);
+	}
+#endif
+
+	dest->height += src->height;
+	dest->nlinks += src->nlinks;
+	dest->links_sorted = 0;
+	sort_links(dest);
+}
+
+void
+remove_document_from_document(struct document *dest, struct document *src, int y)
+{
+	ELOG
+	if (!dest || !src || !src->height) {
+		return;
+	}
+	if (y > dest->height) {
+		y = dest->height;
+	}
+	int i;
+	for (i = 0; i < src->height; i++) {
+		mem_free_if(dest->data[y + i].ch.chars);
+	}
+	memmove(&dest->data[y], &dest->data[y + src->height], (dest->height - src->height - y) * sizeof(struct line));
+
+	if (!ALIGN_LINES(&dest->data, dest->height, dest->height - src->height)) {
+		return;
+	}
+	/* old links */
+
+	for (i = 0; i < dest->nlinks; i++) {
+		struct link *link = &dest->links[i];
+		int c;
+
+		for (c = 0; c < link->npoints; c++) {
+			if (link->points[c].y >= y && (link->points[c].y < (y + src->height))) {
+				link->npoints = 0;
+				break;
+			};
+			if (link->points[c].y >= (y + src->height)) {
+				link->points[c].y -= src->height;
+			}
 		}
 	}
 
-	return -1;
-}
+	int before = dest->nlinks;
+	for (i = 0; i < dest->nlinks; i++) {
+		struct link *link = &dest->links[i];
+
+		if (!link->npoints) {
+			done_link_members(link);
+			memmove(link, link + 1,
+				(dest->nlinks - i - 1) * sizeof(*link));
+			dest->nlinks--;
+			i--;
+		}
+	}
+	//memmove(&dest->links[tomove], &dest->links[tomove + src->nlinks], (src->nlinks) * sizeof(struct link));
+	if (!ALIGN_LINK(&dest->links, before, dest->nlinks)) {
+		return;
+	}
+
+	/* old images */
+#ifdef CONFIG_KITTY
+	struct k_image *k_im, *k_next;
+	foreachsafe (k_im, k_next, dest->k_images) {
+		if (k_im->cy >= y && k_im->cy < (y + src->height)) {
+			delete_k_image(k_im);
+			continue;
+		}
+		k_im->cy -= src->height;
+	}
 #endif
 
+	/* old images */
+#ifdef CONFIG_LIBSIXEL
+	struct image *im, *next;
+	foreachsafe (im, next, dest->images) {
+		if (im->cy >= y && im->cy < (y + src->height)) {
+			del_from_list(im);
+			mem_free(im);
+			continue;
+		}
+		im->cy -= src->height;
+	}
+#endif
+	dest->height -= src->height;
+	//dest->links_sorted = 0;
+	//sort_links(dest);
+}
+
 struct module document_module = struct_module(
 	/* Because this module is listed in main_modules rather than
 	 * in builtin_modules, its name does not appear in the user
