@@ -91,6 +91,7 @@ static void mjs_text_set_property_textContent(js_State *J);
 void *
 mjs_getprivate_text(js_State *J, int idx)
 {
+	ELOG
 	struct mjs_text_private *priv = (struct mjs_text_private *)js_touserdata(J, idx, "text");
 
 	if (!priv) {
@@ -103,10 +104,13 @@ mjs_getprivate_text(js_State *J, int idx)
 static void
 mjs_text_get_property_children(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -127,10 +131,13 @@ mjs_text_get_property_children(js_State *J)
 static void
 mjs_text_get_property_childElementCount(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t res = 0;
@@ -153,10 +160,13 @@ mjs_text_get_property_childElementCount(js_State *J)
 static void
 mjs_text_get_property_childNodes(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -177,10 +187,13 @@ mjs_text_get_property_childNodes(js_State *J)
 static void
 mjs_text_get_property_firstChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -204,10 +217,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_firstElementChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -263,10 +279,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_lastChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *last_child = NULL;
 	dom_exception exc;
 
@@ -290,10 +309,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_lastElementChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -349,6 +371,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_nextElementSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -367,12 +390,10 @@ mjs_text_get_property_nextElementSibling(js_State *J)
 		dom_exception exc = dom_node_get_next_sibling(node, &next);
 		dom_node_type type;
 
-		if (prev_next) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 
 		if (exc != DOM_NO_ERR || !next) {
 			js_pushnull(J);
@@ -398,10 +419,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_nodeName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(node);
+
 	dom_string *name = NULL;
 	dom_exception exc;
 
@@ -422,10 +446,13 @@ mjs_text_get_property_nodeName(js_State *J)
 static void
 mjs_text_get_property_nodeType(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(node);
+
 	dom_node_type type;
 	dom_exception exc;
 
@@ -445,10 +472,13 @@ mjs_text_get_property_nodeType(js_State *J)
 static void
 mjs_text_get_property_nodeValue(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *node = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(node);
+
 	dom_string *content = NULL;
 	dom_exception exc;
 
@@ -469,11 +499,13 @@ mjs_text_get_property_nodeValue(js_State *J)
 static void
 mjs_text_set_property_nodeValue(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *node = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(node);
 
 	if (!node) {
 		js_error(J, "error");
@@ -502,10 +534,13 @@ mjs_text_set_property_nodeValue(js_State *J)
 static void
 mjs_text_get_property_nextSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -529,6 +564,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_ownerDocument(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -540,10 +576,13 @@ mjs_text_get_property_ownerDocument(js_State *J)
 static void
 mjs_text_get_property_parentElement(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -567,10 +606,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_parentNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -595,6 +637,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_previousElementSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -613,12 +656,10 @@ mjs_text_get_property_previousElementSibling(js_State *J)
 		dom_exception exc = dom_node_get_previous_sibling(node, &prev);
 		dom_node_type type;
 
-		if (prev_prev) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_prev);
-		}
+		dom_node_unref(prev_prev);
 
 		if (exc != DOM_NO_ERR || !prev) {
 			js_pushnull(J);
@@ -644,10 +685,13 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_previousSibling(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -671,10 +715,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_get_property_tagName(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -694,10 +740,12 @@ mjs_text_get_property_tagName(js_State *J)
 static void
 mjs_text_get_property_textContent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -717,10 +765,12 @@ mjs_text_get_property_textContent(js_State *J)
 static void
 mjs_text_set_property_textContent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushundefined(J);
@@ -750,6 +800,7 @@ mjs_text_set_property_textContent(js_State *J)
 static xmlpp::Element*
 el_add_child_element_common(xmlNode* child, xmlNode* node)
 {
+	ELOG
 	if (!node) {
 		xmlFreeNode(child);
 		throw xmlpp::internal_error("Could not add child element node");
@@ -763,6 +814,7 @@ el_add_child_element_common(xmlNode* child, xmlNode* node)
 static void
 mjs_text_addEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -773,6 +825,7 @@ mjs_text_addEventListener(js_State *J)
 		return;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -840,9 +893,7 @@ mjs_text_addEventListener(js_State *J)
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(el_private->listener);
 	js_pushundefined(J);
 }
@@ -850,6 +901,7 @@ ex:
 static void
 mjs_text_removeEventListener(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -860,6 +912,7 @@ mjs_text_removeEventListener(js_State *J)
 		return;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -913,11 +966,14 @@ mjs_text_removeEventListener(js_State *J)
 static void
 mjs_text_appendChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *res = NULL;
 
 	if (!el) {
@@ -947,10 +1003,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_cloneNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -976,16 +1034,17 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static bool
 isAncestor(dom_node *el, dom_node *node)
 {
+	ELOG
 	dom_node *prev_next = NULL;
 	while (node) {
 		dom_exception exc;
 		dom_node *next = NULL;
-		if (prev_next) {
+
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
+
 		if (el == node) {
 			return true;
 		}
@@ -1005,10 +1064,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_contains(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1065,14 +1126,16 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 	}
 }
 
-
 static void
 mjs_text_hasChildNodes(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_exception exc;
 	bool res;
 
@@ -1092,11 +1155,13 @@ mjs_text_hasChildNodes(js_State *J)
 static void
 mjs_text_insertBefore(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1127,10 +1192,12 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_isEqualNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1166,10 +1233,12 @@ mjs_text_isEqualNode(js_State *J)
 static void
 mjs_text_isSameNode(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_error(J, "error");
@@ -1183,10 +1252,12 @@ mjs_text_isSameNode(js_State *J)
 static void
 mjs_text_querySelector(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -1214,6 +1285,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_querySelectorAll(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1226,6 +1298,7 @@ mjs_text_querySelectorAll(js_State *J)
 		return;
 	}
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushnull(J);
@@ -1251,11 +1324,14 @@ mjs_text_querySelectorAll(js_State *J)
 static void
 mjs_text_removeChild(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)js_getcontext(J);
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
+
 	dom_node *el2 = (dom_node *)(mjs_getprivate_text(J, 1));
 	dom_exception exc;
 	dom_node *spare;
@@ -1276,6 +1352,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 mjs_text_toString(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1285,17 +1362,16 @@ mjs_text_toString(js_State *J)
 static void
 mjs_text_finalizer(js_State *J, void *priv)
 {
+	ELOG
 	struct mjs_text_private *el_private = (struct mjs_text_private *)priv;
 
 	if (el_private) {
 		dom_node *node = (dom_node *)(el_private->node);
-
-		if (node && (node->refcnt > 0)) {
+		NODEINFO(node);
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(node);
-		}
+		dom_node_unref(node);
 		mem_free(el_private);
 	}
 }
@@ -1303,6 +1379,8 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 void
 mjs_push_text(js_State *J, void *node)
 {
+	ELOG
+	NODEINFO(node);
 	struct mjs_text_private *el_private = NULL;
 
 #if 0
@@ -1338,20 +1416,20 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 		js_copy(J, 0);
 		el_private->thisval = js_ref(J);
 		js_newuserdata(J, "text", el_private, mjs_text_finalizer);
-		addmethod(J, "addEventListener", mjs_text_addEventListener, 3);
-		addmethod(J, "appendChild",mjs_text_appendChild, 1);
-		addmethod(J, "cloneNode",	mjs_text_cloneNode, 1);
-		addmethod(J, "contains",	mjs_text_contains, 1);
-		addmethod(J, "dispatchEvent",	mjs_text_dispatchEvent, 1);
-		addmethod(J, "hasChildNodes",	mjs_text_hasChildNodes, 0);
-		addmethod(J, "insertBefore",	mjs_text_insertBefore, 2);
-		addmethod(J, "isEqualNode",	mjs_text_isEqualNode, 1);
-		addmethod(J, "isSameNode",		mjs_text_isSameNode, 1);
-		addmethod(J, "querySelector",	mjs_text_querySelector, 1);
-		addmethod(J, "querySelectorAll",	mjs_text_querySelectorAll, 1);
-		addmethod(J, "removeChild",	mjs_text_removeChild, 1);
-		addmethod(J, "removeEventListener", mjs_text_removeEventListener, 3);
-		addmethod(J, "toString",		mjs_text_toString, 0);
+		addmethod(J, "Text.prototype.addEventListener", mjs_text_addEventListener, 3);
+		addmethod(J, "Text.prototype.appendChild",mjs_text_appendChild, 1);
+		addmethod(J, "Text.prototype.cloneNode",	mjs_text_cloneNode, 1);
+		addmethod(J, "Text.prototype.contains",	mjs_text_contains, 1);
+		addmethod(J, "Text.prototype.dispatchEvent",	mjs_text_dispatchEvent, 1);
+		addmethod(J, "Text.prototype.hasChildNodes",	mjs_text_hasChildNodes, 0);
+		addmethod(J, "Text.prototype.insertBefore",	mjs_text_insertBefore, 2);
+		addmethod(J, "Text.prototype.isEqualNode",	mjs_text_isEqualNode, 1);
+		addmethod(J, "Text.prototype.isSameNode",		mjs_text_isSameNode, 1);
+		addmethod(J, "Text.prototype.querySelector",	mjs_text_querySelector, 1);
+		addmethod(J, "Text.prototype.querySelectorAll",	mjs_text_querySelectorAll, 1);
+		addmethod(J, "Text.prototype.removeChild",	mjs_text_removeChild, 1);
+		addmethod(J, "Text.prototype.removeEventListener", mjs_text_removeEventListener, 3);
+		addmethod(J, "Text.prototype.toString",		mjs_text_toString, 0);
 
 		addproperty(J, "children",	mjs_text_get_property_children, NULL);
 		addproperty(J, "childElementCount",	mjs_text_get_property_childElementCount, NULL);
@@ -1378,6 +1456,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 int
 mjs_text_init(js_State *J)
 {
+	ELOG
 #if 0
 	mjs_push_node(J, NULL);
 	js_defglobal(J, "text", JS_DONTENUM);
@@ -1389,6 +1468,7 @@ mjs_text_init(js_State *J)
 void
 check_element_event(void *interp, void *elem, const char *event_name, struct term_event *ev)
 {
+	ELOG
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)interp;
 	js_State *J = (js_State *)interpreter->backend_data;
 	void *second = attr_find_in_map(map_privates, elem);
@@ -1426,10 +1506,12 @@ check_element_event(void *interp, void *elem, const char *event_name, struct ter
 static void
 mjs_text_dispatchEvent(js_State *J)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)(mjs_getprivate_text(J, 0));
+	NODEINFO(el);
 
 	if (!el) {
 		js_pushboolean(J, 0);
@@ -1449,6 +1531,7 @@ mjs_text_dispatchEvent(js_State *J)
 static void
 text_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
