@@ -7,7 +7,10 @@
 extern "C" {
 #endif
 
+struct view_state;
+
 int mjs_window_init(js_State *J);
+void mjs_push_window(js_State *J, struct view_state *vs);
 
 #ifdef __cplusplus
 }
