@@ -68,6 +68,7 @@ typedef struct JSString JSString;
 static JSValue
 unicode_to_value(JSContext *ctx, unicode_val_T u)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -96,20 +97,27 @@ unicode_to_value(JSContext *ctx, unicode_val_T u)
 static int
 string_get(const JSString *p, int idx)
 {
+	ELOG
 	return p->is_wide_char ? p->u.str16[idx] : p->u.str8[idx];
 }
 
 /* Convert the string *@vp to an access key.  Return 0 for no access
  * key, UCS_NO_CHAR on error, or the access key otherwise.  */
 static unicode_val_T
-js_value_to_accesskey(JSValueConst val)
+js_value_to_accesskey(JSContext *ctx, JSValueConst val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(val);
+	unicode_val_T ret = UCS_NO_CHAR; /* which the caller will reject */
 
-	JSString *p = JS_VALUE_GET_STRING(val);
+	const char *str = JS_ToCString(ctx, val);
+	JSValue new_val = JS_NewString(ctx, str);
+	JS_FreeCString(ctx, str);
+
+	JSString *p = JS_VALUE_GET_STRING(new_val);
 
 	size_t len;
 	uint16_t chr[2];
@@ -117,26 +125,32 @@ js_value_to_accesskey(JSValueConst val)
 	len = p->len;
 
 	/* This implementation ignores extra characters in the string.  */
-	if (len < 1)
-		return 0;	/* which means no access key */
+	if (len < 1) {
+		ret = 0; /* which means no access key */
+		goto end;
+	}
 	chr[0] = string_get(p, 0);
 
 	if (!is_utf16_surrogate(chr[0])) {
-		return chr[0];
+		ret = chr[0];
+		goto end;
 	}
 	if (len >= 2) {
 		chr[1] = string_get(p, 1);
 		if (is_utf16_high_surrogate(chr[0])
 			&& is_utf16_low_surrogate(chr[1])) {
-			return join_utf16_surrogates(chr[0], chr[1]);
+			ret = join_utf16_surrogates(chr[0], chr[1]);
 		}
 	}
-	return UCS_NO_CHAR;	/* which the caller will reject */
+end:
+	JS_FreeValue(ctx, new_val);
+	return ret;
 }
 
 static JSValue
 js_input_get_property_accessKey(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -193,6 +207,7 @@ js_input_get_property_accessKey(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_accessKey(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -239,7 +254,7 @@ js_input_set_property_accessKey(JSContext *ctx, JSValueConst this_val, JSValue v
 #endif
 		return JS_NULL;
 	}
-	accesskey = js_value_to_accesskey(val);
+	accesskey = js_value_to_accesskey(ctx, val);
 
 	if (link) {
 		link->accesskey = accesskey;
@@ -251,6 +266,7 @@ js_input_set_property_accessKey(JSContext *ctx, JSValueConst this_val, JSValue v
 static JSValue
 js_input_get_property_alt(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -291,6 +307,7 @@ js_input_get_property_alt(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_alt(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -341,6 +358,7 @@ js_input_set_property_alt(JSContext *ctx, JSValueConst this_val, JSValue val)
 static JSValue
 js_input_get_property_checked(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -361,6 +379,7 @@ js_input_get_property_checked(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_checked(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -406,6 +425,7 @@ js_input_set_property_checked(JSContext *ctx, JSValueConst this_val, JSValue val
 static JSValue
 js_input_get_property_defaultChecked(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -445,6 +465,7 @@ js_input_get_property_defaultChecked(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_defaultValue(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -484,6 +505,7 @@ js_input_get_property_defaultValue(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_disabled(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -525,6 +547,7 @@ js_input_get_property_disabled(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_disabled(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -571,6 +594,7 @@ js_input_set_property_disabled(JSContext *ctx, JSValueConst this_val, JSValue va
 static JSValue
 js_input_get_property_form(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -598,6 +622,7 @@ js_input_get_property_form(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_maxLength(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -638,6 +663,7 @@ js_input_get_property_maxLength(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_maxLength(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -681,6 +707,7 @@ js_input_set_property_maxLength(JSContext *ctx, JSValueConst this_val, JSValue v
 static JSValue
 js_input_get_property_name(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -723,6 +750,7 @@ js_input_get_property_name(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_name(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -774,6 +802,7 @@ js_input_set_property_name(JSContext *ctx, JSValueConst this_val, JSValue val)
 static JSValue
 js_input_get_property_readonly(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -816,6 +845,7 @@ js_input_get_property_readonly(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_readonly(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -862,6 +892,7 @@ js_input_set_property_readonly(JSContext *ctx, JSValueConst this_val, JSValue va
 static JSValue
 js_input_get_property_selectedIndex(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -907,6 +938,7 @@ js_input_get_property_selectedIndex(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_selectedIndex(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -943,7 +975,7 @@ js_input_set_property_selectedIndex(JSContext *ctx, JSValueConst this_val, JSVal
 	assert(fc->form && fs);
 
 	if (fc->type == FC_SELECT) {
-		int item;
+		int32_t item;
 
 		JS_ToInt32(ctx, &item, val);
 
@@ -959,6 +991,7 @@ js_input_set_property_selectedIndex(JSContext *ctx, JSValueConst this_val, JSVal
 static JSValue
 js_input_get_property_size(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -999,6 +1032,7 @@ js_input_get_property_size(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_src(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1049,6 +1083,7 @@ js_input_get_property_src(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_src(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1109,6 +1144,7 @@ js_input_set_property_src(JSContext *ctx, JSValueConst this_val, JSValue val)
 static JSValue
 js_input_get_property_tabIndex(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1160,6 +1196,7 @@ js_input_get_property_tabIndex(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_type(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1217,6 +1254,7 @@ js_input_get_property_type(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_get_property_value(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1239,6 +1277,7 @@ js_input_get_property_value(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_input_set_property_value(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1300,6 +1339,7 @@ js_input_set_property_value(JSContext *ctx, JSValueConst this_val, JSValue val)
 static struct form_state *
 js_input_get_form_state(JSContext *ctx, JSValueConst jsinput)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1314,6 +1354,7 @@ js_input_get_form_state(JSContext *ctx, JSValueConst jsinput)
 static JSValue
 js_input_blur(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1328,6 +1369,7 @@ js_input_blur(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *arg
 static JSValue
 js_input_click(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1379,6 +1421,7 @@ js_input_click(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *ar
 static JSValue
 js_input_focus(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1424,6 +1467,7 @@ js_input_focus(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *ar
 static JSValue
 js_input_select(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1437,6 +1481,7 @@ js_input_select(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *a
 JSValue
 js_get_input_object(JSContext *ctx, struct form_state *fs)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1446,6 +1491,7 @@ js_get_input_object(JSContext *ctx, struct form_state *fs)
 static JSValue
 js_input_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1482,6 +1528,7 @@ static const JSCFunctionListEntry js_input_proto_funcs[] = {
 void
 quickjs_detach_form_state(struct form_state *fs)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1497,6 +1544,7 @@ quickjs_detach_form_state(struct form_state *fs)
 void
 quickjs_moved_form_state(struct form_state *fs)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1513,6 +1561,7 @@ quickjs_moved_form_state(struct form_state *fs)
 static
 void js_input_finalizer(JSRuntime *rt, JSValue val)
 {
+	ELOG
 	REF_JS(val);
 
 	struct form_state *fs = (struct form_state *)JS_GetOpaque(val, js_input_class_id);
@@ -1531,6 +1580,7 @@ static JSClassDef js_input_class = {
 static JSValue
 js_input_ctor(JSContext *ctx, JSValueConst new_target, int argc, JSValueConst *argv)
 {
+	ELOG
 	REF_JS(new_target);
 
 	JSValue obj = JS_UNDEFINED;
@@ -1559,6 +1609,7 @@ fail:
 int
 js_input_init(JSContext *ctx, JSValue global_obj)
 {
+	ELOG
 	REF_JS(global_obj);
 
 	JSValue input_proto, input_class;
@@ -1586,6 +1637,7 @@ js_input_init(JSContext *ctx, JSValue global_obj)
 JSValue
 getInput(JSContext *ctx, struct form_state *fs)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
