@@ -73,6 +73,7 @@ static JSValue js_text_set_property_textContent(JSContext *ctx, JSValueConst thi
 void *
 js_getopaque_text(JSValueConst obj, JSClassID class_id)
 {
+	ELOG
 	//REF_JS(obj);
 
 	struct js_text_private *res = (struct js_text_private *)JS_GetOpaque(obj, class_id);
@@ -93,18 +94,22 @@ js_getopaque_text(JSValueConst obj, JSClassID class_id)
 void *
 text_get_node(JSValueConst obj)
 {
+	ELOG
 	return js_getopaque_text(obj, js_text_class_id);
 }
 
 static JSValue
 js_text_get_property_children(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -127,12 +132,15 @@ js_text_get_property_children(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_childElementCount(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t res = 0;
@@ -157,12 +165,15 @@ js_text_get_property_childElementCount(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_childNodes(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -182,11 +193,10 @@ js_text_get_property_childNodes(JSContext *ctx, JSValueConst this_val)
 	RETURN_JS(rr);
 }
 
-
-
 static JSValue
 js_text_get_property_firstChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -195,6 +205,7 @@ js_text_get_property_firstChild(JSContext *ctx, JSValueConst this_val)
 	dom_exception exc;
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -218,12 +229,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_firstElementChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -282,12 +296,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_lastChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *last_child = NULL;
 	dom_exception exc;
 
@@ -313,12 +330,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_lastElementChild(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -376,12 +396,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_nextElementSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_next = NULL;
 
@@ -396,12 +419,10 @@ js_text_get_property_nextElementSibling(JSContext *ctx, JSValueConst this_val)
 		dom_exception exc = dom_node_get_next_sibling(node, &next);
 		dom_node_type type;
 
-		if (prev_next) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 
 		if (exc != DOM_NO_ERR || !next) {
 			return JS_NULL;
@@ -427,12 +448,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_nodeName(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(node);
+
 	dom_string *name = NULL;
 	dom_exception exc;
 	JSValue r;
@@ -460,12 +484,15 @@ js_text_get_property_nodeName(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(node);
+
 	dom_node_type type;
 	dom_exception exc;
 
@@ -487,12 +514,15 @@ js_text_get_property_nodeType(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_nodeValue(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *node = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(node);
+
 	dom_string *content = NULL;
 	dom_exception exc;
 	JSValue r;
@@ -517,12 +547,14 @@ js_text_get_property_nodeValue(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_set_property_nodeValue(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *node = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(node);
 
 	if (!node) {
 		return JS_UNDEFINED;
@@ -553,12 +585,15 @@ js_text_set_property_nodeValue(JSContext *ctx, JSValueConst this_val, JSValue va
 static JSValue
 js_text_get_property_nextSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -582,11 +617,10 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 	return rr;
 }
 
-
-
 static JSValue
 js_text_get_property_ownerDocument(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -601,12 +635,15 @@ js_text_get_property_ownerDocument(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_parentElement(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)js_getopaque_text(this_val, js_text_class_id);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -632,11 +669,14 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_parentNode(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	dom_node *el = (dom_node *)js_getopaque_text(this_val, js_text_class_id);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -663,12 +703,15 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_previousElementSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_prev = NULL;
 
@@ -683,12 +726,10 @@ js_text_get_property_previousElementSibling(JSContext *ctx, JSValueConst this_va
 		dom_exception exc = dom_node_get_previous_sibling(node, &prev);
 		dom_node_type type;
 
-		if (prev_prev) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_prev);
-		}
+		dom_node_unref(prev_prev);
 
 		if (exc != DOM_NO_ERR || !prev) {
 			return JS_NULL;
@@ -715,11 +756,14 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_previousSibling(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -746,6 +790,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_get_property_tagName(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -753,6 +798,7 @@ js_text_get_property_tagName(JSContext *ctx, JSValueConst this_val)
 	JSValue r;
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -776,12 +822,14 @@ js_text_get_property_tagName(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_get_property_textContent(JSContext *ctx, JSValueConst this_val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	REF_JS(this_val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -804,6 +852,7 @@ js_text_get_property_textContent(JSContext *ctx, JSValueConst this_val)
 static JSValue
 js_text_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -811,6 +860,7 @@ js_text_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSValue
 	REF_JS(val);
 
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_UNDEFINED;
@@ -844,6 +894,7 @@ js_text_set_property_textContent(JSContext *ctx, JSValueConst this_val, JSValue
 static xmlpp::Element*
 el_add_child_element_common(xmlNode* child, xmlNode* node)
 {
+	ELOG
 	if (!node) {
 		xmlFreeNode(child);
 		throw xmlpp::internal_error("Could not add child element node");
@@ -857,6 +908,7 @@ el_add_child_element_common(xmlNode* child, xmlNode* node)
 static JSValue
 js_text_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -868,6 +920,7 @@ js_text_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 		return JS_FALSE;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_FALSE;
@@ -899,6 +952,7 @@ js_text_dispatchEvent(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 static JSValue
 js_text_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -910,6 +964,7 @@ js_text_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValu
 		return JS_NULL;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -982,9 +1037,7 @@ js_text_addEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValu
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(el_private->listener);
 	//dom_node_unref(el);
 
@@ -994,6 +1047,7 @@ ex:
 static JSValue
 js_text_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1005,6 +1059,7 @@ js_text_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSV
 		return JS_NULL;
 	}
 	dom_node *el = el_private->node;
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1068,6 +1123,7 @@ js_text_removeEventListener(JSContext *ctx, JSValueConst this_val, int argc, JSV
 static JSValue
 js_text_appendChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1075,6 +1131,8 @@ js_text_appendChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_node *res = NULL;
 	dom_exception exc;
 
@@ -1117,6 +1175,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_cloneNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1129,6 +1188,7 @@ js_text_cloneNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 		return JS_NULL;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_NULL;
@@ -1158,16 +1218,17 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static bool
 isAncestor(dom_node *el, dom_node *node)
 {
+	ELOG
 	dom_node *prev_next = NULL;
 	while (node) {
 		dom_exception exc;
 		dom_node *next = NULL;
-		if (prev_next) {
+
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
+
 		if (el == node) {
 			return true;
 		}
@@ -1186,6 +1247,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_contains(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1198,6 +1260,7 @@ js_text_contains(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1248,6 +1311,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static JSValue
 js_text_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1260,6 +1324,8 @@ js_text_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
+
 	dom_exception exc;
 	bool res;
 
@@ -1281,6 +1347,7 @@ js_text_hasChildNodes(JSContext *ctx, JSValueConst this_val, int argc, JSValueCo
 static JSValue
 js_text_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1294,6 +1361,7 @@ js_text_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1335,6 +1403,7 @@ js_text_insertBefore(JSContext *ctx, JSValueConst this_val, int argc, JSValueCon
 static JSValue
 js_text_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1347,6 +1416,7 @@ js_text_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1390,6 +1460,7 @@ js_text_isEqualNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 static JSValue
 js_text_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1402,6 +1473,7 @@ js_text_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 		return JS_UNDEFINED;
 	}
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el) {
 		return JS_EXCEPTION;
@@ -1421,6 +1493,7 @@ js_text_isSameNode(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst
 static JSValue
 js_text_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1434,6 +1507,7 @@ js_text_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 	}
 	struct ecmascript_interpreter *interpreter = (struct ecmascript_interpreter *)JS_GetContextOpaque(ctx);
 	dom_node *el = (dom_node *)(js_getopaque_text(this_val, js_text_class_id));
+	NODEINFO(el);
 
 	if (!el || !JS_IsObject(argv[0])) {
 		return JS_NULL;
@@ -1460,6 +1534,7 @@ js_text_removeChild(JSContext *ctx, JSValueConst this_val, int argc, JSValueCons
 static JSValue
 js_text_toString(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1505,6 +1580,7 @@ static const JSCFunctionListEntry js_text_proto_funcs[] = {
 static
 void js_text_finalizer(JSRuntime *rt, JSValue val)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1515,20 +1591,16 @@ void js_text_finalizer(JSRuntime *rt, JSValue val)
 	if (el_private) {
 		struct text_listener *l;
 		dom_node *el = (dom_node *)el_private->node;
+		NODEINFO(el);
 
 		if (el_private->listener) {
 			dom_event_listener_unref(el_private->listener);
 		}
 
-		if (el) {
-
-			if (el->refcnt > 0) {
 #ifdef ECMASCRIPT_DEBUG
 fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 #endif
-				dom_node_unref(el);
-			}
-		}
+		dom_node_unref(el);
 
 		foreach(l, el_private->listeners) {
 			mem_free_set(&l->typ, NULL);
@@ -1544,6 +1616,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 js_text_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
@@ -1571,6 +1644,7 @@ static JSClassDef js_text_class = {
 int
 js_text_init(JSContext *ctx)
 {
+	ELOG
 	JSValue element_proto;
 
 	/* create the element class */
@@ -1595,9 +1669,11 @@ js_text_init(JSContext *ctx)
 JSValue
 getText(JSContext *ctx, void *node)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
+	NODEINFO(node);
 	struct js_text_private *el_private = (struct js_text_private *)mem_calloc(1, sizeof(*el_private));
 
 	if (!el_private) {
@@ -1626,6 +1702,7 @@ fprintf(stderr, "Before: %s:%d\n", __FUNCTION__, __LINE__);
 static void
 text_event_handler(dom_event *event, void *pw)
 {
+	ELOG
 #ifdef ECMASCRIPT_DEBUG
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
