@@ -4,10 +4,11 @@
 
 #include "js/spidermonkey/util.h"
 
+struct view_state;
+
 extern JSClass location_class;
 extern const spidermonkeyFunctionSpec location_funcs[];
 extern JSPropertySpec location_props[];
-JSObject *getLocation(JSContext *ctx);
-
+JSObject *getLocation(JSContext *ctx, struct view_state *vs);
 
 #endif
