@@ -161,6 +161,8 @@ static void text_finalize(JS::GCContext *op, JSObject *obj)
 	fprintf(stderr, "%s:%s\n", __FILE__, __FUNCTION__);
 #endif
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(obj, 0);
+	NODEINFO(el);
+
 	struct text_private *el_private = JS::GetMaybePtrFromReservedSlot<struct text_private>(obj, 1);
 
 	if (el_private) {
@@ -183,10 +185,8 @@ static void text_finalize(JS::GCContext *op, JSObject *obj)
 		}
 	}
 
-	if (el) {
-		dom_node_unref(el);
-		JS::SetReservedSlot(obj, 0, JS::UndefinedValue());
-	}
+	dom_node_unref(el);
+	JS::SetReservedSlot(obj, 0, JS::UndefinedValue());
 }
 
 
@@ -230,6 +230,8 @@ text_get_property_children(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -289,6 +291,8 @@ text_get_property_childElementCount(JSContext *ctx, unsigned int argc, JS::Value
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t res = 0;
@@ -350,6 +354,8 @@ text_get_property_childNodes(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 
@@ -413,6 +419,7 @@ text_get_property_firstChild(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	dom_exception exc;
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -472,6 +479,8 @@ text_get_property_firstElementChild(JSContext *ctx, unsigned int argc, JS::Value
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -562,6 +571,8 @@ text_get_property_lastChild(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *last_child = NULL;
 	dom_exception exc;
 
@@ -623,6 +634,8 @@ text_get_property_lastElementChild(JSContext *ctx, unsigned int argc, JS::Value
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_nodelist *nodes = NULL;
 	dom_exception exc;
 	uint32_t size = 0;
@@ -712,6 +725,8 @@ text_get_property_nextElementSibling(JSContext *ctx, unsigned int argc, JS::Valu
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_next = NULL;
 
@@ -726,9 +741,7 @@ text_get_property_nextElementSibling(JSContext *ctx, unsigned int argc, JS::Valu
 		dom_exception exc = dom_node_get_next_sibling(node, &next);
 		dom_node_type type;
 
-		if (prev_next) {
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 
 		if (exc != DOM_NO_ERR || !next) {
 			args.rval().setNull();
@@ -790,7 +803,9 @@ text_get_property_nodeName(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 
-	dom_node *node = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);;
+	dom_node *node = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(node);
+
 	dom_string *name = NULL;
 	dom_exception exc;
 
@@ -850,6 +865,8 @@ text_get_property_nodeType(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *node = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(node);
+
 	dom_node_type type1;
 	dom_exception exc;
 
@@ -908,6 +925,8 @@ text_get_property_nodeValue(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *node = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(node);
+
 	dom_string *content = NULL;
 	dom_exception exc;
 
@@ -966,6 +985,8 @@ text_set_property_nodeValue(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *node = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(node);
+
 	args.rval().setUndefined();
 
 	if (!node) {
@@ -1030,6 +1051,8 @@ text_get_property_nextSibling(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -1133,6 +1156,8 @@ text_get_property_parentElement(JSContext *ctx, unsigned int argc, JS::Value *vp
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -1193,6 +1218,8 @@ text_get_property_parentNode(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -1254,6 +1281,8 @@ text_get_property_previousElementSibling(JSContext *ctx, unsigned int argc, JS::
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node;
 	dom_node *prev_prev = NULL;
 
@@ -1268,9 +1297,7 @@ text_get_property_previousElementSibling(JSContext *ctx, unsigned int argc, JS::
 		dom_exception exc = dom_node_get_previous_sibling(node, &prev);
 		dom_node_type type;
 
-		if (prev_prev) {
-			dom_node_unref(prev_prev);
-		}
+		dom_node_unref(prev_prev);
 
 		if (exc != DOM_NO_ERR || !prev) {
 			args.rval().setNull();
@@ -1333,6 +1360,8 @@ text_get_property_previousSibling(JSContext *ctx, unsigned int argc, JS::Value *
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *node = NULL;
 	dom_exception exc;
 
@@ -1393,6 +1422,7 @@ text_get_property_tagName(JSContext *ctx, unsigned int argc, JS::Value *vp)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -1450,6 +1480,7 @@ text_get_property_textContent(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -1499,6 +1530,7 @@ text_set_property_textContent(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setUndefined();
@@ -1595,6 +1627,8 @@ text_addEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_node *el = JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	struct text_private *el_private = JS::GetMaybePtrFromReservedSlot<struct text_private>(hobj, 1);
 
 	if (!el || !el_private) {
@@ -1654,9 +1688,7 @@ text_addEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 ex:
-	if (typ) {
-		dom_string_unref(typ);
-	}
+	dom_string_unref(typ);
 	dom_event_listener_unref(el_private->listener);
 	args.rval().setUndefined();
 
@@ -1689,6 +1721,8 @@ text_removeEventListener(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_node *el = JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	struct text_private *el_private = JS::GetMaybePtrFromReservedSlot<struct text_private>(hobj, 1);
 
 	if (!el || !el_private) {
@@ -1768,6 +1802,8 @@ text_appendChild(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_node *res = NULL;
 
 	if (argc != 1) {
@@ -1827,6 +1863,7 @@ text_cloneNode(JSContext *ctx, unsigned int argc, JS::Value *rval)
 	}
 
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -1855,9 +1892,7 @@ isAncestor(dom_node *el, dom_node *node)
 	while (node) {
 		dom_exception exc;
 		dom_node *next = NULL;
-		if (prev_next) {
-			dom_node_unref(prev_next);
-		}
+		dom_node_unref(prev_next);
 		if (el == node) {
 			return true;
 		}
@@ -1899,6 +1934,7 @@ text_contains(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		return false;
@@ -1961,6 +1997,8 @@ text_hasChildNodes(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return true;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
+
 	dom_exception exc;
 	bool res;
 
@@ -2006,6 +2044,7 @@ text_insertBefore(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		return false;
@@ -2066,6 +2105,7 @@ text_isEqualNode(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		return false;
@@ -2129,6 +2169,7 @@ text_isSameNode(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		return false;
@@ -2165,6 +2206,7 @@ text_querySelector(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -2220,6 +2262,7 @@ text_querySelectorAll(JSContext *ctx, unsigned int argc, JS::Value *vp)
 		return true;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el) {
 		args.rval().setNull();
@@ -2277,6 +2320,7 @@ text_removeChild(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *el = (dom_node *)JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(el);
 
 	if (!el || !args[0].isObject()) {
 		args.rval().setNull();
@@ -2406,6 +2450,7 @@ text_dispatchEvent(JSContext *ctx, unsigned int argc, JS::Value *rval)
 		return false;
 	}
 	dom_node *element = JS::GetMaybePtrFromReservedSlot<dom_node>(hobj, 0);
+	NODEINFO(element);
 
 	if (!element) {
 		args.rval().setBoolean(false);
