@@ -48,6 +48,13 @@ struct mailcap_hash_item {
 	char type[1];
 };
 
+enum {
+	MAILCAP_KIND_NORMAL,
+	MAILCAP_KIND_COPIOUSOUTPUT,
+	MAILCAP_KIND_X_ANSIOUTPUT,
+	MAILCAP_KIND_X_HTMLOUTPUT
+};
+
 struct mailcap_entry {
 	LIST_HEAD_EL(struct mailcap_entry);
 
@@ -64,10 +71,8 @@ struct mailcap_entry {
 	/* Whether the program "blocks" the term. */
 	unsigned int needsterminal:1;
 
-	/* If "| ${PAGER}" should be added. It would of course be better to
-	 * pipe the output into a buffer and let ELinks display it but this
-	 * will have to do for now. */
-	unsigned int copiousoutput:1;
+	/* normal, copiousoutput, x_ansioutput or x_htmloutput */
+	unsigned char kind;
 
 	/* The 'raw' unformatted (view)command from the mailcap files. */
 	char command[1];
@@ -147,6 +152,7 @@ static struct hash *mailcap_map = NULL;
 static inline void
 done_mailcap_entry(struct mailcap_entry *entry)
 {
+	ELOG
 	if (!entry) return;
 	mem_free_if(entry->testcommand);
 	mem_free_if(entry->description);
@@ -159,6 +165,7 @@ done_mailcap_entry(struct mailcap_entry *entry)
 static inline struct mailcap_entry *
 init_mailcap_entry(char *command, int priority)
 {
+	ELOG
 	struct mailcap_entry *entry;
 	int commandlen = strlen(command);
 
@@ -175,6 +182,7 @@ init_mailcap_entry(char *command, int priority)
 static inline void
 add_mailcap_entry(struct mailcap_entry *entry, char *type, int typelen)
 {
+	ELOG
 	struct mailcap_hash_item *mitem;
 	struct hash_item *item;
 
@@ -227,6 +235,7 @@ add_mailcap_entry(struct mailcap_entry *entry, char *type, int typelen)
 static char *
 get_mailcap_field(char **next)
 {
+	ELOG
 	char *field;
 	char *fieldend;
 
@@ -271,6 +280,7 @@ get_mailcap_field(char **next)
 static char *
 get_mailcap_field_text(char *field)
 {
+	ELOG
 	skip_space(field);
 
 	if (*field == '=') {
@@ -287,6 +297,7 @@ get_mailcap_field_text(char *field)
 static inline int
 parse_optional_fields(struct mailcap_entry *entry, char *line)
 {
+	ELOG
 	while (0xf131d5) {
 		char *field = get_mailcap_field(&line);
 
@@ -296,7 +307,13 @@ parse_optional_fields(struct mailcap_entry *entry, char *line)
 			entry->needsterminal = 1;
 
 		} else if (!c_strncasecmp(field, "copiousoutput", 13)) {
-			entry->copiousoutput = 1;
+			entry->kind = MAILCAP_KIND_COPIOUSOUTPUT;
+
+		} else if (!c_strncasecmp(field, "x-ansioutput", 12)) {
+			entry->kind = MAILCAP_KIND_X_ANSIOUTPUT;
+
+		} else if (!c_strncasecmp(field, "x-htmloutput", 12)) {
+			entry->kind = MAILCAP_KIND_X_HTMLOUTPUT;
 
 		} else if (!c_strncasecmp(field, "test", 4)) {
 			/* Don't leak memory if a corrupted mailcap
@@ -331,6 +348,7 @@ parse_optional_fields(struct mailcap_entry *entry, char *line)
 static void
 parse_mailcap_file(char *filename, unsigned int priority)
 {
+	ELOG
 	FILE *file = fopen(filename, "rb");
 	char *line = NULL;
 	size_t linelen = MAX_STR_LEN;
@@ -409,6 +427,7 @@ parse_mailcap_file(char *filename, unsigned int priority)
 static struct hash *
 init_mailcap_map(void)
 {
+	ELOG
 	char *path;
 	unsigned int priority = 0;
 
@@ -434,6 +453,7 @@ init_mailcap_map(void)
 static void
 done_mailcap(struct module *module)
 {
+	ELOG
 	struct hash_item *item;
 	int i;
 
@@ -462,6 +482,7 @@ done_mailcap(struct module *module)
 static int
 change_hook_mailcap(struct session *ses, struct option *current, struct option *changed)
 {
+	ELOG
 	if (changed == &get_opt_mailcap(MAILCAP_PATH)
 	    || (changed == &get_opt_mailcap(MAILCAP_ENABLE)
 		&& !get_mailcap_enable())) {
@@ -474,6 +495,7 @@ change_hook_mailcap(struct session *ses, struct option *current, struct option *
 static void
 init_mailcap(struct module *module)
 {
+	ELOG
 	static const struct change_hook_info mimetypes_change_hooks[] = {
 		{ "mime.mailcap",		change_hook_mailcap },
 		{ NULL,				NULL },
@@ -507,8 +529,9 @@ init_mailcap(struct module *module)
 /* The formatting is postponed until the command is needed. This means
  * @type can be NULL. If '%t' is used in command we bail out. */
 static char *
-format_command(char *command, char *type, int copiousoutput)
+format_command(char *command, char *type, int kind)
 {
+	ELOG
 	struct string cmd;
 
 	if (!init_string(&cmd)) return NULL;
@@ -571,23 +594,34 @@ format_command(char *command, char *type, int copiousoutput)
 static struct mailcap_entry *
 check_entries(struct mailcap_hash_item *item)
 {
+	ELOG
 	struct mailcap_entry *entry;
 
-	foreach (entry, item->entries) {
-		char *test;
+	unsigned char kind;
 
-		/* Accept current if no test is needed */
-		if (!entry->testcommand)
-			return entry;
+	for (kind = MAILCAP_KIND_X_HTMLOUTPUT; kind >= MAILCAP_KIND_NORMAL; kind--) {
+		foreach (entry, item->entries) {
+			char *test;
 
-		/* We have to run the test command */
-		test = format_command(entry->testcommand, NULL, 0);
-		if (test) {
-			int exitcode = exe(test);
+			if (entry->kind != kind) {
+				continue;
+			}
 
-			mem_free(test);
-			if (!exitcode)
+			/* Accept current if no test is needed */
+			if (!entry->testcommand) {
 				return entry;
+			}
+
+			/* We have to run the test command */
+			test = format_command(entry->testcommand, NULL, 0);
+			if (test) {
+				int exitcode = exe(test);
+
+				mem_free(test);
+				if (!exitcode) {
+					return entry;
+				}
+			}
 		}
 	}
 
@@ -608,6 +642,7 @@ check_entries(struct mailcap_hash_item *item)
 static struct mailcap_entry *
 get_mailcap_entry(char *type)
 {
+	ELOG
 	struct mailcap_entry *entry;
 	struct hash_item *item;
 
@@ -691,6 +726,7 @@ get_mailcap_entry(char *type)
 static void
 set_display(int xwin, int restore)
 {
+	ELOG
 	static char *display = NULL;
 
 	if (!restore) {
@@ -742,6 +778,7 @@ set_display(int xwin, int restore)
 static struct mime_handler *
 get_mime_handler_mailcap(char *type, int xwin)
 {
+	ELOG
 	struct mailcap_entry *entry;
 	struct mime_handler *handler;
 	char *program;
@@ -761,16 +798,17 @@ get_mime_handler_mailcap(char *type, int xwin)
 #endif
 	if (!entry) return NULL;
 
-	program = format_command(entry->command, type, entry->copiousoutput);
+	program = format_command(entry->command, type, entry->kind);
 	if (!program) return NULL;
 
-	block = (entry->needsterminal || entry->copiousoutput);
+	block = (entry->needsterminal || entry->kind);
 	handler = init_mime_handler(program, entry->description,
 				    mailcap_mime_module.name,
 				    get_mailcap_ask(), block);
 	mem_free(program);
 
-	handler->copiousoutput = entry->copiousoutput;
+	handler->copiousoutput = ((entry->kind == MAILCAP_KIND_COPIOUSOUTPUT) || (entry->kind == MAILCAP_KIND_X_ANSIOUTPUT));
+	handler->x_htmloutput = (entry->kind == MAILCAP_KIND_X_HTMLOUTPUT);
 	return handler;
 }
 
@@ -802,12 +840,14 @@ struct option *cmdline_options;
 union option_value *
 get_opt_(char *a, int, enum option_type b, struct option *c, const char *d, struct session *e)
 {
+	ELOG
 	return NULL;
 }
 #else
 union option_value *
 get_opt_(struct option *a , const char *b, struct session *c)
 {
+	ELOG
 	return NULL;
 }
 #endif
@@ -816,14 +856,19 @@ char *get_ui_clipboard_file(void) { return NULL; }
 
 /* Some ugly shortcuts for getting defined symbols to work. */
 int default_mime_backend,
-    install_signal_handler,
     mimetypes_mime_backend,
     program;
 LIST_OF(struct terminal) terminals;
 
+void
+install_signal_handler(int sig, void (*fn)(void *), void *data, int critical)
+{
+}
+
 int
 main(int argc, char *argv[])
 {
+	ELOG
 	char *format = "description,ask,block,program";
 	int has_gotten = 0;
 	int setenv_display = 0;
