@@ -4,6 +4,9 @@
 #include "config.h"
 #endif
 
+#include <libguile.h>
+#include <stdio.h>
+
 #include "elinks.h"
 
 #include "intl/libintl.h"
@@ -14,7 +17,10 @@
 static const char *
 get_name_guile(struct module *xxx)
 {
+	ELOG
 	static char elguileversion[32];
+
+	scm_init_guile();
 	snprintf(elguileversion, 31, "Guile %s", scm_to_locale_string(scm_version()));
 
 	return elguileversion;
