@@ -4,6 +4,12 @@
 #include "config.h"
 #endif
 
+#undef _GNU_SOURCE
+#include <ruby.h>
+#include <ruby/version.h>
+
+#undef _
+
 #include "elinks.h"
 
 #include "intl/libintl.h"
@@ -14,6 +20,7 @@
 static const char *
 get_name_ruby(struct module *xxx)
 {
+	ELOG
 	static char elrubyversion[32];
 	snprintf(elrubyversion, 31, "Ruby %s", ruby_version);
 
