@@ -1029,6 +1029,29 @@ move_cursor_line_start(struct session *ses, struct document_view *doc_view)
 	return move_cursor_rel(ses, doc_view, -x, 0);
 }
 
+enum frame_event_status
+move_cursor_line_end(struct session *ses, struct document_view *doc_view)
+{
+	ELOG
+	struct view_state *vs;
+	struct el_box *box;
+	int x, y, xend;
+
+	assert(ses && doc_view && doc_view->vs && doc_view->document);
+	if_assert_failed return FRAME_EVENT_OK;
+
+	vs = doc_view->vs;
+	box = &doc_view->box;
+	x = vs->x + ses->tab->x - box->x;
+	y = vs->y + ses->tab->y - box->y;
+
+	if (y >= doc_view->document->height) {
+		return FRAME_EVENT_OK;
+	}
+	xend = int_max(doc_view->document->data[y].length - 1, 0);
+	return move_cursor_rel(ses, doc_view, xend - x, 0);
+}
+
 static enum frame_event_status
 move_clipboard_pos(struct session *ses, struct document_view *view, enum frame_event_status status)
 {
@@ -1090,8 +1113,13 @@ copy_to_clipboard2(struct document_view *doc_view)
 	utf8 = document->options.utf8;
 #endif
 
+	int remove_leading_space = get_opt_bool("ui.clipboard.remove_leading_spaces", NULL);
+	int remove_trailing_space = get_opt_bool("ui.clipboard.remove_trailing_spaces", NULL);
+
 	for (y = starty; y <= endy; y++) {
 		int ex = int_min(endx, document->data[y].length - 1);
+		int rls = remove_leading_space;
+		int space_counter = 0;
 		int x;
 
 		for (x = startx; x <= ex; x++) {
@@ -1119,7 +1147,18 @@ copy_to_clipboard2(struct document_view *doc_view)
 #else
 			if (!isscreensafe(c)) c = ' ';
 #endif
-
+			if (rls && (c == ' ')) {
+				continue;
+			} else {
+				rls = 0;
+			}
+			if (remove_trailing_space && (c == ' ')) {
+				space_counter++;
+				continue;
+			}
+			for (; space_counter; space_counter--) {
+				add_char_to_string(&data, ' ');
+			}
 #ifdef CONFIG_UTF8
 			if (utf8) {
 				add_to_string(&data, encode_utf8(c));
@@ -1130,6 +1169,11 @@ copy_to_clipboard2(struct document_view *doc_view)
 			add_char_to_string(&data, c);
 #endif
 		}
+		if (!remove_trailing_space) {
+			for (; space_counter; space_counter--) {
+				add_char_to_string(&data, ' ');
+			}
+		}
 		add_char_to_string(&data, '\n');
 	}
 
