@@ -32,14 +32,20 @@
 void
 init_vs(struct view_state *vs, struct uri *uri, int plain)
 {
+	ELOG
 	memset(vs, 0, sizeof(*vs));
 	vs->current_link = -1;
 	vs->old_current_link = -1;
 	vs->current_search_number = -1;
 	vs->plain = plain;
+	vs->wrap = get_opt_bool("document.browse.wrap", NULL);
 	vs->uri = uri ? get_uri_reference(uri) : NULL;
 	vs->did_fragment = !uri->fragmentlen;
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+#ifdef CONFIG_QUICKJS
+	vs->win_obj = JS_NULL;
+	vs->location_obj = JS_NULL;
+#endif
 	/* If we ever get to render this vs, give it an interpreter. */
 	vs->ecmascript_fragile = 1;
 #endif
@@ -50,6 +56,7 @@ init_vs(struct view_state *vs, struct uri *uri, int plain)
 void
 destroy_vs(struct view_state *vs, int blast_ecmascript)
 {
+	ELOG
 	struct form_view *fv, *next;
 
 #ifdef CONFIG_SCRIPTING_SPIDERMONKEY
@@ -69,6 +76,14 @@ destroy_vs(struct view_state *vs, int blast_ecmascript)
 	
 	if (vs->uri) done_uri(vs->uri);
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+#ifdef CONFIG_ECMASCRIPT_SMJS
+	detach_js_view_state(vs);
+#endif
+
+#ifdef CONFIG_QUICKJS
+	detach_js_view_state(vs);
+#endif
+
 	if (blast_ecmascript && vs->ecmascript) {
 		ecmascript_put_interpreter(vs->ecmascript);
 	}
@@ -83,6 +98,7 @@ destroy_vs(struct view_state *vs, int blast_ecmascript)
 void
 copy_vs(struct view_state *dst, struct view_state *src)
 {
+	ELOG
 	struct form_view *fv;
 
 	copy_struct(dst, src);
@@ -90,6 +106,14 @@ copy_vs(struct view_state *dst, struct view_state *src)
 	/* We do not copy ecmascript stuff around since it's specific for
 	 * a single location, offsprings (followups and so) nedd their own. */
 #if defined(CONFIG_ECMASCRIPT_SMJS) || defined(CONFIG_QUICKJS) || defined(CONFIG_MUJS)
+#ifdef CONFIG_ECMASCRIPT_SMJS
+	dst->winobject = NULL;
+	dst->locobject = NULL;
+#endif
+#ifdef CONFIG_QUICKJS
+	dst->win_obj = JS_NULL;
+	dst->location_obj = JS_NULL;
+#endif
 	dst->ecmascript = NULL;
 	/* If we ever get to render this vs, give it an interpreter. */
 	dst->ecmascript_fragile = 1;
@@ -154,6 +178,7 @@ copy_vs(struct view_state *dst, struct view_state *src)
 void
 check_vs(struct document_view *doc_view)
 {
+	ELOG
 	struct view_state *vs = doc_view->vs;
 
 	int_upper_bound(&vs->current_link, doc_view->document->nlinks - 1);
@@ -173,6 +198,7 @@ check_vs(struct document_view *doc_view)
 void
 next_frame(struct session *ses, int p)
 {
+	ELOG
 	struct view_state *vs;
 	struct document_view *doc_view;
 	int n;
